@@ -25,6 +25,35 @@
 #include <lsmtraits.h>
 #include <string.h>
 
+static const char *lsm_svg_blending_mode_strings[] = {
+	"normal",
+	"multiply",
+	"screen",
+	"darken",
+	"lighten",
+	"over",
+	"in",
+	"out",
+	"atop",
+	"xor"
+};
+
+const char *
+lsm_svg_blending_mode_to_string (LsmSvgBlendingMode blending_mode)
+{
+	if (blending_mode < 0 || blending_mode > LSM_SVG_BLENDING_MODE_LIGHTEN)
+		return NULL;
+
+	return lsm_svg_blending_mode_strings[blending_mode];
+}
+
+LsmSvgBlendingMode
+lsm_svg_blending_mode_from_string (const char *string)
+{
+	return lsm_enum_value_from_string (string, lsm_svg_blending_mode_strings,
+					   G_N_ELEMENTS (lsm_svg_blending_mode_strings));
+}
+
 static const char *lsm_svg_length_type_strings[] = {
 	"",
 	"%",
@@ -119,6 +148,31 @@ lsm_svg_line_cap_from_string (const char *string)
 					   G_N_ELEMENTS (lsm_svg_line_cap_strings));
 }
 
+static const char *lsm_svg_overflow_strings[] = {
+	"visible",
+	"hidden",
+	"scroll",
+	"auto"
+};
+
+const char *
+lsm_svg_overflow_to_string (LsmSvgOverflow overflow)
+{
+	if (overflow < 0 || overflow > LSM_SVG_OVERFLOW_AUTO)
+		return NULL;
+
+	return lsm_svg_overflow_strings[overflow];
+}
+
+LsmSvgOverflow
+lsm_svg_overflow_from_string (const char *string)
+{
+	LsmSvgOverflow overflow = lsm_enum_value_from_string (string, lsm_svg_overflow_strings,
+							      G_N_ELEMENTS (lsm_svg_overflow_strings));
+
+	return overflow;
+}
+
 static const char *lsm_svg_pattern_units_strings[] = {
 	"userSpaceOnUse",
 	"objectBoundingBox"
@@ -141,8 +195,8 @@ lsm_svg_pattern_units_from_string (const char *string)
 }
 
 static const char *lsm_svg_marker_units_strings[] = {
-	"userSpaceOnUse",
-	"strokeWidth"
+	"strokeWidth",
+	"userSpaceOnUse"
 };
 
 const char *
@@ -442,3 +496,28 @@ lsm_svg_visibility_from_string (const char *string)
 					   G_N_ELEMENTS (lsm_svg_visibility_strings));
 }
 
+static const char *lsm_svg_writing_mode_strings[] = {
+	"lr-tb",
+	"rl-tb",
+	"tb-rl",
+	"lr",
+	"rl",
+	"tb"
+};
+
+const char *
+lsm_svg_writing_mode_to_string (LsmSvgWritingMode writing_mode)
+{
+	if (writing_mode < 0 || writing_mode > LSM_SVG_WRITING_MODE_TB)
+		return NULL;
+
+	return lsm_svg_writing_mode_strings[writing_mode];
+}
+
+LsmSvgWritingMode
+lsm_svg_writing_mode_from_string (const char *string)
+{
+	return lsm_enum_value_from_string (string, lsm_svg_writing_mode_strings,
+					   G_N_ELEMENTS (lsm_svg_writing_mode_strings));
+}
+
