@@ -32,6 +32,56 @@ lsm_svg_angle_type_get_type (void)
 	return the_type;
 }
 
+GType
+lsm_svg_blending_mode_get_type (void)
+{
+	static GType the_type = 0;
+
+	if (the_type == 0)
+	{
+		static const GEnumValue values[] = {
+			{ LSM_SVG_BLENDING_MODE_ERROR,
+			  "LSM_SVG_BLENDING_MODE_ERROR",
+			  "error" },
+			{ LSM_SVG_BLENDING_MODE_NORMAL,
+			  "LSM_SVG_BLENDING_MODE_NORMAL",
+			  "normal" },
+			{ LSM_SVG_BLENDING_MODE_MULTIPLY,
+			  "LSM_SVG_BLENDING_MODE_MULTIPLY",
+			  "multiply" },
+			{ LSM_SVG_BLENDING_MODE_SCREEN,
+			  "LSM_SVG_BLENDING_MODE_SCREEN",
+			  "screen" },
+			{ LSM_SVG_BLENDING_MODE_DARKEN,
+			  "LSM_SVG_BLENDING_MODE_DARKEN",
+			  "darken" },
+			{ LSM_SVG_BLENDING_MODE_LIGHTEN,
+			  "LSM_SVG_BLENDING_MODE_LIGHTEN",
+			  "lighten" },
+			{ LSM_SVG_BLENDING_MODE_OVER,
+			  "LSM_SVG_BLENDING_MODE_OVER",
+			  "over" },
+			{ LSM_SVG_BLENDING_MODE_IN,
+			  "LSM_SVG_BLENDING_MODE_IN",
+			  "in" },
+			{ LSM_SVG_BLENDING_MODE_OUT,
+			  "LSM_SVG_BLENDING_MODE_OUT",
+			  "out" },
+			{ LSM_SVG_BLENDING_MODE_ATOP,
+			  "LSM_SVG_BLENDING_MODE_ATOP",
+			  "atop" },
+			{ LSM_SVG_BLENDING_MODE_XOR,
+			  "LSM_SVG_BLENDING_MODE_XOR",
+			  "xor" },
+			{ 0, NULL, NULL }
+		};
+		the_type = g_enum_register_static (
+				g_intern_static_string ("LsmSvgBlendingMode"),
+				values);
+	}
+	return the_type;
+}
+
 GType
 lsm_svg_paint_type_get_type (void)
 {
@@ -283,6 +333,38 @@ lsm_svg_transform_type_get_type (void)
 	return the_type;
 }
 
+GType
+lsm_svg_overflow_get_type (void)
+{
+	static GType the_type = 0;
+
+	if (the_type == 0)
+	{
+		static const GEnumValue values[] = {
+			{ LSM_SVG_OVERFLOW_ERROR,
+			  "LSM_SVG_OVERFLOW_ERROR",
+			  "error" },
+			{ LSM_SVG_OVERFLOW_VISIBLE,
+			  "LSM_SVG_OVERFLOW_VISIBLE",
+			  "visible" },
+			{ LSM_SVG_OVERFLOW_HIDDEN,
+			  "LSM_SVG_OVERFLOW_HIDDEN",
+			  "hidden" },
+			{ LSM_SVG_OVERFLOW_SCROLL,
+			  "LSM_SVG_OVERFLOW_SCROLL",
+			  "scroll" },
+			{ LSM_SVG_OVERFLOW_AUTO,
+			  "LSM_SVG_OVERFLOW_AUTO",
+			  "auto" },
+			{ 0, NULL, NULL }
+		};
+		the_type = g_enum_register_static (
+				g_intern_static_string ("LsmSvgOverflow"),
+				values);
+	}
+	return the_type;
+}
+
 GType
 lsm_svg_pattern_units_get_type (void)
 {
@@ -320,12 +402,12 @@ lsm_svg_marker_units_get_type (void)
 			{ LSM_SVG_MARKER_UNITS_ERROR,
 			  "LSM_SVG_MARKER_UNITS_ERROR",
 			  "error" },
-			{ LSM_SVG_MARKER_UNITS_USER_SPACE_ON_USE,
-			  "LSM_SVG_MARKER_UNITS_USER_SPACE_ON_USE",
-			  "user-space-on-use" },
 			{ LSM_SVG_MARKER_UNITS_STROKE_WIDTH,
 			  "LSM_SVG_MARKER_UNITS_STROKE_WIDTH",
 			  "stroke-width" },
+			{ LSM_SVG_MARKER_UNITS_USER_SPACE_ON_USE,
+			  "LSM_SVG_MARKER_UNITS_USER_SPACE_ON_USE",
+			  "user-space-on-use" },
 			{ 0, NULL, NULL }
 		};
 		the_type = g_enum_register_static (
@@ -857,6 +939,44 @@ lsm_svg_element_category_get_type (void)
 	return the_type;
 }
 
+GType
+lsm_svg_writing_mode_get_type (void)
+{
+	static GType the_type = 0;
+
+	if (the_type == 0)
+	{
+		static const GEnumValue values[] = {
+			{ LSM_SVG_WRITING_MODE_ERROR,
+			  "LSM_SVG_WRITING_MODE_ERROR",
+			  "error" },
+			{ LSM_SVG_WRITING_MODE_LR_TB,
+			  "LSM_SVG_WRITING_MODE_LR_TB",
+			  "lr-tb" },
+			{ LSM_SVG_WRITING_MODE_RL_TB,
+			  "LSM_SVG_WRITING_MODE_RL_TB",
+			  "rl-tb" },
+			{ LSM_SVG_WRITING_MODE_TB_RL,
+			  "LSM_SVG_WRITING_MODE_TB_RL",
+			  "tb-rl" },
+			{ LSM_SVG_WRITING_MODE_LR,
+			  "LSM_SVG_WRITING_MODE_LR",
+			  "lr" },
+			{ LSM_SVG_WRITING_MODE_RL,
+			  "LSM_SVG_WRITING_MODE_RL",
+			  "rl" },
+			{ LSM_SVG_WRITING_MODE_TB,
+			  "LSM_SVG_WRITING_MODE_TB",
+			  "tb" },
+			{ 0, NULL, NULL }
+		};
+		the_type = g_enum_register_static (
+				g_intern_static_string ("LsmSvgWritingMode"),
+				values);
+	}
+	return the_type;
+}
+
 /* enumerations from "lsmsvgview.h" */
 #include "lsmsvgview.h"
 
