@@ -31,6 +31,30 @@
 const LsmSvgColor lsm_svg_color_null = {0.0, 0.0, 0.0};
 const LsmSvgDashArray lsm_svg_dash_array_null = {0, NULL};
 
+static gboolean
+lsm_svg_blending_mode_trait_from_string (LsmTrait *abstract_trait, char *string)
+{
+	LsmSvgBlendingMode *trait = (LsmSvgBlendingMode *) abstract_trait;
+
+	*trait = lsm_svg_blending_mode_from_string (string);
+
+	return *trait >= 0;
+}
+
+char *
+lsm_svg_blending_mode_trait_to_string (LsmTrait *abstract_trait)
+{
+	LsmSvgBlendingMode *trait = (LsmSvgBlendingMode *) abstract_trait;
+
+	return g_strdup (lsm_svg_blending_mode_to_string (*trait));
+}
+
+const LsmTraitClass lsm_svg_blending_mode_trait_class = {
+	.size = sizeof (LsmSvgBlendingMode),
+	.from_string = lsm_svg_blending_mode_trait_from_string,
+	.to_string = lsm_svg_blending_mode_trait_to_string
+};
+
 static gboolean
 lsm_svg_length_trait_from_string (LsmTrait *abstract_trait, char *string)
 {
@@ -936,6 +960,8 @@ lsm_svg_one_or_two_double_trait_from_string (LsmTrait *abstract_trait, char *str
 		return FALSE;
 	}
 
+	string = end_ptr;
+
 	lsm_str_skip_spaces (&string);
 
 	if (string[0] == '\0') {
@@ -965,3 +991,51 @@ const LsmTraitClass lsm_svg_one_or_two_double_trait_class = {
 	.to_string = lsm_svg_one_or_two_double_trait_to_string
 };
 
+static gboolean
+lsm_svg_overflow_trait_from_string (LsmTrait *abstract_trait, char *string)
+{
+	LsmSvgOverflow *trait = (LsmSvgOverflow *) abstract_trait;
+
+	*trait = lsm_svg_overflow_from_string (string);
+
+	return *trait >= 0;
+}
+
+char *
+lsm_svg_overflow_trait_to_string (LsmTrait *abstract_trait)
+{
+	LsmSvgOverflow *trait = (LsmSvgOverflow *) abstract_trait;
+
+	return g_strdup (lsm_svg_overflow_to_string (*trait));
+}
+
+const LsmTraitClass lsm_svg_overflow_trait_class = {
+	.size = sizeof (LsmSvgOverflow),
+	.from_string = lsm_svg_overflow_trait_from_string,
+	.to_string = lsm_svg_overflow_trait_to_string
+};
+
+static gboolean
+lsm_svg_writing_mode_trait_from_string (LsmTrait *abstract_trait, char *string)
+{
+	LsmSvgDisplay *trait = (LsmSvgDisplay *) abstract_trait;
+
+	*trait = lsm_svg_writing_mode_from_string (string);
+
+	return *trait >= 0;
+}
+
+char *
+lsm_svg_writing_mode_trait_to_string (LsmTrait *abstract_trait)
+{
+	LsmSvgDisplay *trait = (LsmSvgDisplay *) abstract_trait;
+
+	return g_strdup (lsm_svg_writing_mode_to_string (*trait));
+}
+
+const LsmTraitClass lsm_svg_writing_mode_trait_class = {
+	.size = sizeof (LsmSvgWritingMode),
+	.from_string = lsm_svg_writing_mode_trait_from_string,
+	.to_string = lsm_svg_writing_mode_trait_to_string
+};
+
