@@ -27,6 +27,7 @@
 #include <lsmsvgelement.h>
 #include <lsmsvgsvgelement.h>
 #include <lsmsvgradialgradientelement.h>
+#include <lsmsvgfilterelement.h>
 #include <lsmsvglineargradientelement.h>
 #include <lsmsvgpatternelement.h>
 #include <lsmsvgmarkerelement.h>
@@ -64,6 +65,7 @@ struct _LsmSvgViewPatternData {
 	cairo_pattern_t *pattern;
 
 	LsmBox extents;
+	LsmBox object_extents;
 
 	double opacity;
 };
@@ -71,22 +73,13 @@ struct _LsmSvgViewPatternData {
 double
 lsm_svg_view_normalize_length (LsmSvgView *view, const LsmSvgLength *length, LsmSvgLengthDirection direction)
 {
-	double font_size;
-
 	g_return_val_if_fail (LSM_IS_SVG_VIEW (view), 0.0);
 
-	/* TODO cache font size */
-	if (view->viewbox_stack != NULL && view->style != NULL) {
-		font_size = lsm_svg_length_normalize (&view->style->font_size->length, view->viewbox_stack->data,
-						      0.0, LSM_SVG_LENGTH_DIRECTION_DIAGONAL);
-	} else
-		font_size = 0.0;
-
-	return lsm_svg_length_normalize (length, view->viewbox_stack->data, font_size, direction);
+	return lsm_svg_length_normalize (length, view->viewbox_stack->data, view->style->font_size_px, direction);
 }
 
 static void
-_start_pattern (LsmSvgView *view, const LsmBox *extents, double opacity)
+_start_pattern (LsmSvgView *view, const LsmBox *extents, const LsmBox *object_extents, double opacity)
 {
 	lsm_debug_render ("[LsmSvgView::start_pattern]");
 
@@ -97,6 +90,7 @@ _start_pattern (LsmSvgView *view, const LsmBox *extents, double opacity)
 	view->pattern_data->pattern = NULL;
 	view->pattern_data->extents = *extents;
 	view->pattern_data->opacity = opacity;
+	view->pattern_data->object_extents = *object_extents;
 
 	view->dom_view.cairo = NULL;
 }
@@ -298,7 +292,7 @@ lsm_svg_view_create_surface_pattern (LsmSvgView *view,
 	x_scale = device_width / viewport->width;
 	y_scale = device_height / viewport->height;
 
-	lsm_debug_render ("[LsmSvgView::create_pattern] pattern size = %g ,%g at %g, %g (scale %g x %g)",
+	lsm_debug_render ("[LsmSvgView::create_surface_pattern] pattern size = %g ,%g at %g, %g (scale %g x %g)",
 		   device_width, device_height, viewport->x, viewport->y, x_scale, y_scale);
 
 	switch (surface_type) {
@@ -345,6 +339,10 @@ lsm_svg_view_create_surface_pattern (LsmSvgView *view,
 	cairo_pattern_set_matrix (view->pattern_data->pattern, &matrix);
 	cairo_pattern_set_extend (view->pattern_data->pattern, CAIRO_EXTEND_REPEAT);
 
+	lsm_debug_render ("[LsmSvgView::create_surface_pattern] Pattern matrix %g, %g, %g, %g, %g, %g",
+			  matrix.xx, matrix.xy, matrix.yx, matrix.yy,
+			  matrix.x0, matrix.y0);
+
 	return TRUE;
 }
 
@@ -788,8 +786,14 @@ _paint_url (LsmSvgView *view,
 	if ((!LSM_IS_SVG_RADIAL_GRADIENT_ELEMENT (element) &&
 	     !LSM_IS_SVG_LINEAR_GRADIENT_ELEMENT (element) &&
 	     !LSM_IS_SVG_PATTERN_ELEMENT (element)) ||
-	    lsm_svg_view_circular_reference_check (view, element))
+	    lsm_svg_view_circular_reference_check (view, element)) {
+
+		cairo_set_source_rgba (view->dom_view.cairo, 0, 0, 0, 0);
+
+		lsm_warning_render ("[LsmSvgView::_paint_url] Paint url not found: %s", url);
+
 		return;
+	}
 
 	lsm_debug_render ("[LsmSvgView::_paint_url] Paint using '%s'", url);
 
@@ -810,26 +814,24 @@ _paint_url (LsmSvgView *view,
 	lsm_debug_render ("[LsmSvgView::_paint_url] Pattern extents x = %g, y = %g, w = %g, h = %g",
 		   extents.x, extents.y, extents.width, extents.height);
 
-	_start_pattern (view, &extents, opacity);
+	_start_pattern (view, &extents, &extents, opacity);
 
 	lsm_svg_element_force_render (LSM_SVG_ELEMENT (element), view);
 
 	cairo = view->pattern_data->old_cairo;
 
 	if (view->pattern_data->pattern) {
-#if 0
-		if (LSM_IS_SVG_PATTERN_ELEMENT (element)) {
+		if (view->debug_pattern && view->dom_view.cairo) {
 			char *filename;
 
-			filename = g_strdup_printf ("pattern-%s.png", url);
+			filename = g_strdup_printf ("pattern-%s).png", url);
 			cairo_surface_write_to_png (cairo_get_target (view->dom_view.cairo), filename);
 			g_free (filename);
 		}
-#endif
 
 		cairo_set_source (cairo, view->pattern_data->pattern);
 	} else
-		cairo_set_source_rgb (cairo, 0.0, 0.0, 0.0);
+		cairo_set_source_rgba (cairo, 0.0, 0.0, 0.0, 0.0);
 
 	_end_pattern (view);
 }
@@ -978,17 +980,27 @@ paint_markers (LsmSvgView *view)
 			} else if (next_data == NULL ||
 				   next_data->header.type == CAIRO_PATH_MOVE_TO) {
 				marker = marker_end;
-				if (type == CAIRO_PATH_CURVE_TO)
-					angle = atan2 (y - data[2].point.y,
-						       x - data[2].point.x);
-				else
+				if (type == CAIRO_PATH_CURVE_TO) {
+					if (y != data[2].point.y ||
+					    x != data[2].point.x)
+						angle = atan2 (y - data[2].point.y,
+							       x - data[2].point.x);
+					else
+						angle = atan2 (y - data[1].point.y,
+							       x - data[1].point.x);
+				} else
 					angle = atan2 (y - prev_y, x - prev_x);
 			} else if (data->header.type == CAIRO_PATH_MOVE_TO) {
 				marker = marker_start;
-				if (next_type == CAIRO_PATH_CURVE_TO)
-					angle = atan2 (data[1].point.y - y,
-						       data[1].point.x - x);
-				else
+				if (next_type == CAIRO_PATH_CURVE_TO) {
+					if (next_data[1].point.y != y ||
+					    next_data[1].point.x != x)
+						angle = atan2 (next_data[1].point.y - y,
+							       next_data[1].point.x - x);
+					else
+						angle = atan2 (next_data[2].point.y - y,
+							       next_data[2].point.x - x);
+				} else
 					angle = atan2 (next_y - y, next_x - x);
 			} else {
 				double xdifin, ydifin, xdifout, ydifout, intot, outtot;
@@ -996,15 +1008,27 @@ paint_markers (LsmSvgView *view)
 				marker = marker_mid;
 
 				if (type == CAIRO_PATH_CURVE_TO) {
-					xdifin = x - data[2].point.x;
-					ydifin = y - data[2].point.y;
+					if (x != data[2].point.x ||
+					    y != data[2].point.y) {
+						xdifin = x - data[2].point.x;
+						ydifin = y - data[2].point.y;
+					} else {
+						xdifin = x - data[1].point.x;
+						ydifin = y - data[1].point.y;
+					}
 				} else {
 					xdifin = x - prev_x;
 					ydifin = y - prev_y;
 				}
 				if (next_type == CAIRO_PATH_CURVE_TO) {
-					xdifout = data[3].point.x - x;
-					ydifout = data[3].point.y - y;
+					if (next_data[1].point.x != x ||
+					    next_data[1].point.y != y) {
+						xdifout = next_data[1].point.x - x;
+						ydifout = next_data[1].point.y - y;
+					} else {
+						xdifout = next_data[2].point.x - x;
+						ydifout = next_data[2].point.y - y;
+					}
 				} else {
 					xdifout = next_x - x;
 					ydifout = next_y - y;
@@ -1050,6 +1074,9 @@ paint (LsmSvgView *view, LsmSvgViewPathInfos *path_infos)
 	cairo = view->dom_view.cairo;
 	style = view->style;
 
+#if 0
+	/* This is disabled for now, as care should be taken in push_composition to
+	 * defer group opacity for simple shape to this function. */
 	if (style->opacity != NULL) {
 		group_opacity = style->opacity->value;
 
@@ -1060,8 +1087,12 @@ paint (LsmSvgView *view, LsmSvgViewPathInfos *path_infos)
 		use_group = FALSE;
 		group_opacity = 1.0;
 	}
+#else
+	use_group = FALSE;
+	group_opacity = 1.0;
+#endif
 
-	/* FIXME Instead of push_group, restrict to the current path bounding box */
+	/* Instead of push_group, we should restrict to the current path bounding box */
 	if (use_group)
 		cairo_push_group (cairo);
 
@@ -1359,19 +1390,18 @@ lsm_svg_view_show_text (LsmSvgView *view, char const *string, double x, double y
 	PangoStyle font_style;
 	PangoLayoutIter *iter;
 	PangoRectangle rectangle;
-	int font_size;
 	int baseline;
 	double x1, y1;
 
-	if (string == NULL)
+	if (string == NULL || string[0] == '\0')
 		return;
 
 	g_return_if_fail (LSM_IS_SVG_VIEW (view));
 
-	lsm_debug_render ("[LsmSvgView::show_text] Show '%s' at %g,%g", string, x, y);
-
 	style = view->style;
 
+	lsm_debug_render ("[LsmSvgView::show_text] Show '%s' at %g,%g (%g px)", string, x, y, style->font_size_px);
+
 	if (view->is_pango_layout_in_use) {
 		PangoContext *pango_context;
 
@@ -1386,14 +1416,8 @@ lsm_svg_view_show_text (LsmSvgView *view, char const *string, double x, double y
 	pango_layout = view->pango_layout;
 	font_description = view->dom_view.font_description;
 
-	font_size = PANGO_SCALE * lsm_svg_view_normalize_length (view, &style->font_size->length,
-								 LSM_SVG_LENGTH_DIRECTION_DIAGONAL);
-
-	if (font_size < 0)
-		font_size = 0;
-
 	pango_font_description_set_family (font_description, style->font_family->value);
-	pango_font_description_set_size (font_description, font_size);
+	pango_font_description_set_size (font_description, PANGO_SCALE * style->font_size_px);
 	pango_font_description_set_weight (font_description, style->font_weight->value);
 
 	switch (style->font_stretch->value) {
@@ -1465,8 +1489,6 @@ lsm_svg_view_show_text (LsmSvgView *view, char const *string, double x, double y
 			break;
 	}
 
-	cairo_move_to (view->dom_view.cairo, x1, y1);
-
 	path_infos.is_text_path = TRUE;
 	path_infos.is_extents_defined = TRUE;
 	path_infos.extents.x1 = x1;
@@ -1475,7 +1497,20 @@ lsm_svg_view_show_text (LsmSvgView *view, char const *string, double x, double y
 	path_infos.extents.y2 = y1 + pango_units_to_double (rectangle.height);
 	path_infos.pango_layout = pango_layout;
 
-	process_path (view, &path_infos);
+	if (style->writing_mode->value == LSM_SVG_WRITING_MODE_TB ||
+	    style->writing_mode->value == LSM_SVG_WRITING_MODE_TB_RL) {
+
+		cairo_save (view->dom_view.cairo);
+		cairo_rotate (view->dom_view.cairo, M_PI / 2.0);
+		cairo_move_to (view->dom_view.cairo, x1, y1);
+
+		process_path (view, &path_infos);
+
+		cairo_restore (view->dom_view.cairo);
+	} else {
+		cairo_move_to (view->dom_view.cairo, x1, y1);
+		process_path (view, &path_infos);
+	}
 
 	if (pango_layout != view->pango_layout) {
 		lsm_debug_render ("[LsmSvgView::show_text] Free the child pango layout");
@@ -1616,7 +1651,8 @@ _compute_viewbox_scale (const LsmBox *viewport, const LsmBox *viewbox,
 
 void
 lsm_svg_view_push_viewport (LsmSvgView *view, const LsmBox *viewport, const LsmBox *viewbox,
-			    const LsmSvgPreserveAspectRatio *aspect_ratio)
+			    const LsmSvgPreserveAspectRatio *aspect_ratio,
+			    LsmSvgOverflow overflow)
 {
 	cairo_t *cairo;
 	const LsmBox *actual_viewbox;
@@ -1633,8 +1669,8 @@ lsm_svg_view_push_viewport (LsmSvgView *view, const LsmBox *viewport, const LsmB
 	cairo = view->dom_view.cairo;
 
 	cairo_save (cairo);
-#if 0
-	if (view->dom_view.debug) {
+
+	if (lsm_debug_check (&lsm_debug_category_viewport, LSM_DEBUG_LEVEL_LOG)) {
 		cairo_save (cairo);
 		cairo_set_line_width (cairo, 1.0);
 		cairo_set_source_rgb (cairo, 0.0, 0.0, 0.0);
@@ -1642,9 +1678,12 @@ lsm_svg_view_push_viewport (LsmSvgView *view, const LsmBox *viewport, const LsmB
 		cairo_stroke (cairo);
 		cairo_restore (cairo);
 	}
-#endif
-	cairo_rectangle (cairo, viewport->x, viewport->y, viewport->width, viewport->height);
-	cairo_clip (cairo);
+
+	if (overflow == LSM_SVG_OVERFLOW_HIDDEN) {
+		cairo_rectangle (cairo, viewport->x, viewport->y, viewport->width, viewport->height);
+		cairo_clip (cairo);
+	}
+
 	cairo_translate (cairo, viewport->x + x_offset, viewport->y + y_offset);
 	cairo_scale (cairo, x_scale, y_scale);
 }
@@ -1740,28 +1779,6 @@ lsm_svg_view_pop_matrix (LsmSvgView *view)
 	}
 }
 
-void
-lsm_svg_view_push_group_opacity (LsmSvgView *view)
-{
-	g_return_if_fail (LSM_IS_SVG_VIEW (view));
-	g_return_if_fail (view->style != NULL);
-
-	if (view->style->opacity->value < 1.0)
-		cairo_push_group (view->dom_view.cairo);
-}
-
-void
-lsm_svg_view_pop_group_opacity (LsmSvgView *view)
-{
-	g_return_if_fail (LSM_IS_SVG_VIEW (view));
-	g_return_if_fail (view->style != NULL);
-
-	if (view->style->opacity->value < 1.0) {
-		cairo_pop_group_to_source (view->dom_view.cairo);
-		cairo_paint_with_alpha (view->dom_view.cairo, view->style->opacity->value);
-	}
-}
-
 static void
 lsm_svg_view_push_clip (LsmSvgView *view)
 {
@@ -1785,6 +1802,12 @@ lsm_svg_view_push_clip (LsmSvgView *view)
 	view->clip_extents.width  = extents.x2 - extents.x1;
 	view->clip_extents.height = extents.y2 - extents.y1;
 
+	lsm_debug_render ("[LsmSvgView::push_clip] x=%g y=%g w=%g h=%g",
+			  view->clip_extents.x,
+			  view->clip_extents.y,
+			  view->clip_extents.width,
+			  view->clip_extents.height);
+
 	element = lsm_svg_document_get_element_by_url (LSM_SVG_DOCUMENT (view->dom_view.document), url);
 	if (LSM_IS_SVG_CLIP_PATH_ELEMENT (element) &&
 	    !lsm_svg_view_circular_reference_check (view, element)) {
@@ -1792,7 +1815,8 @@ lsm_svg_view_push_clip (LsmSvgView *view)
 		lsm_svg_element_force_render (LSM_SVG_ELEMENT (element), view);
 		cairo_clip (view->dom_view.cairo);
 		view->is_clipping = FALSE;
-	}
+	} else
+		lsm_warning_render ("[LsmSvgView::push_clip] Clip path not found: %s", view->style->clip_path->value);
 }
 
 static void
@@ -1836,7 +1860,7 @@ lsm_svg_view_pop_mask (LsmSvgView *view)
 
 		cairo = view->dom_view.cairo;
 
-		_start_pattern (view, &mask_extents, 1.0);
+		_start_pattern (view, &mask_extents, &mask_extents, 1.0);
 
 		lsm_svg_element_force_render (LSM_SVG_ELEMENT (mask_element), view);
 
@@ -1864,15 +1888,15 @@ lsm_svg_view_pop_mask (LsmSvgView *view)
 			}
 
 			cairo_pattern_set_extend (view->pattern_data->pattern, CAIRO_EXTEND_NONE);
-#if 0
-			{
+
+			if (view->debug_mask && view->dom_view.cairo != NULL) {
 				char *filename;
 
 				filename = g_strdup_printf ("mask-%s.png", view->style->mask->value);
 				cairo_surface_write_to_png (cairo_get_target (view->dom_view.cairo), filename);
 				g_free (filename);
 			}
-#endif
+
 			cairo_mask (cairo, view->pattern_data->pattern);
 		} else {
 			cairo_paint (cairo);
@@ -1880,6 +1904,8 @@ lsm_svg_view_pop_mask (LsmSvgView *view)
 
 		_end_pattern (view);
 	} else {
+		lsm_warning_render ("[LsmSvgView::pop_mask] Mask url nout found: %s", view->style->mask->value);
+
 		cairo_pop_group_to_source (view->dom_view.cairo);
 		cairo_paint (view->dom_view.cairo);
 	}
@@ -1888,38 +1914,57 @@ lsm_svg_view_pop_mask (LsmSvgView *view)
 void
 lsm_svg_view_push_filter (LsmSvgView *view)
 {
-#if 0
 	LsmExtents extents;
-	LsmBox source_extents;
-	gboolean result;
+	LsmBox object_extents;
+	LsmBox effect_viewport;
+	LsmSvgElement *filter_element;
+	gboolean success;
 
 	g_return_if_fail (LSM_IS_SVG_VIEW (view));
 	g_return_if_fail (view->element_stack != NULL);
 
 	lsm_svg_element_get_extents (view->element_stack->data, view, &extents);
 
-	source_extents.x = extents.x1;
-	source_extents.y = extents.y1;
-	source_extents.width = extents.x2 - extents.x1;
-	source_extents.height = extents.y2 - extents.y1;
+	object_extents.x = extents.x1;
+	object_extents.y = extents.y1;
+	object_extents.width = extents.x2 - extents.x1;
+	object_extents.height = extents.y2 - extents.y1;
 
-	_start_pattern (view, &source_extents, 1.0);
+	filter_element = lsm_svg_document_get_element_by_url (LSM_SVG_DOCUMENT (view->dom_view.document),
+							      view->style->filter->value);
 
-	result = lsm_svg_view_create_surface_pattern (view,
-						      &source_extents,
-						      NULL,
-						      LSM_SVG_VIEW_SURFACE_TYPE_IMAGE);
-#endif
+	if (LSM_IS_SVG_FILTER_ELEMENT (filter_element)) {
+		effect_viewport = lsm_svg_filter_element_get_effect_viewport (LSM_SVG_FILTER_ELEMENT (filter_element),
+									      &object_extents, view);
+
+		_start_pattern (view, &effect_viewport, &object_extents,
+				view->style->opacity != NULL ? view->style->opacity->value : 1.0);
+
+		success = lsm_svg_view_create_surface_pattern (view,
+							      &effect_viewport,
+							      NULL,
+							      LSM_SVG_VIEW_SURFACE_TYPE_IMAGE);
+	} else {
+		lsm_warning_render ("LsmSvgView::push_filter] Filter not found: %s", view->style->filter->value);
+
+		_start_pattern (view, &object_extents, &object_extents, 0.0);
+
+		success = lsm_svg_view_create_surface_pattern (view,
+							       &object_extents,
+							       NULL,
+							       LSM_SVG_VIEW_SURFACE_TYPE_IMAGE);
+	}
+
+	if (!success) 
+		lsm_warning_render ("LsmSvgView::push_filter] Failed to create subsurface");
 }
 
 void
 lsm_svg_view_pop_filter (LsmSvgView *view)
 {
-#if 0
 	LsmSvgElement *filter_element;
 	LsmFilterSurface *filter_surface;
 	cairo_surface_t *surface;
-	cairo_t *cairo;
 	GSList *iter;
 
 	g_return_if_fail (LSM_IS_SVG_VIEW (view));
@@ -1927,41 +1972,163 @@ lsm_svg_view_pop_filter (LsmSvgView *view)
 	filter_element = lsm_svg_document_get_element_by_url (LSM_SVG_DOCUMENT (view->dom_view.document),
 							      view->style->filter->value);
 
-TODO: lsm_svg_view_circular_reference_check
+	if (LSM_IS_SVG_FILTER_ELEMENT (filter_element) &&
+	    view->pattern_data->pattern != NULL) {
+		cairo_matrix_t matrix;
 
-	cairo = view->pattern_data->old_cairo;
+		view->filter_surfaces = NULL;
 
-	view->filter_surfaces = NULL;
+		cairo_pattern_get_surface (view->pattern_data->pattern, &surface);
+		filter_surface = lsm_filter_surface_new_with_content ("SourceGraphic", 0, 0, surface);
+		cairo_pattern_get_matrix (view->pattern_data->pattern, &matrix);
 
-	cairo_pattern_get_surface (view->pattern_data->pattern, &surface);
-	filter_surface = lsm_filter_surface_new_with_content ("SourceGraphic", 0, 0, surface);
+		view->filter_surfaces = g_slist_prepend (view->filter_surfaces, filter_surface);
 
-	view->filter_surfaces = g_slist_prepend (view->filter_surfaces, filter_surface);
+		lsm_svg_element_force_render (filter_element, view);
 
-#if 1
-	{
-		char *filename;
-		static int count = 0;
+		if (view->debug_filter) {
+			GSList *iter;
+			char *filename;
+			static int count = 0;
 
-		filename = g_strdup_printf ("filter-%04d-%s.png", count++, view->style->filter->value);
-		cairo_surface_write_to_png (cairo_get_target (view->dom_view.cairo), filename);
-		g_free (filename);
+			for (iter = view->filter_surfaces; iter != NULL; iter = iter->next) {
+				LsmFilterSurface *surface = iter->data;
+
+				filename = g_strdup_printf ("filter-%04d-%s-%s.png", count++,
+							    view->style->filter->value,
+							    lsm_filter_surface_get_name (surface));
+				cairo_surface_write_to_png (lsm_filter_surface_get_cairo_surface (surface), filename);
+				g_free (filename);
+			}
+		}
+
+		if (view->filter_surfaces->next != NULL) {
+			cairo_pattern_t *pattern;
+			cairo_surface_t *surface;
+
+			surface = lsm_filter_surface_get_cairo_surface (view->filter_surfaces->data);
+			pattern = cairo_pattern_create_for_surface (surface);
+			cairo_pattern_set_extend (pattern, CAIRO_EXTEND_NONE);
+			cairo_pattern_set_matrix (pattern, &matrix);
+			cairo_set_source (view->pattern_data->old_cairo, pattern);
+			cairo_paint_with_alpha (view->pattern_data->old_cairo, view->style->opacity->value);
+		}
+
+		for (iter = view->filter_surfaces; iter != NULL; iter = iter->next)
+			lsm_filter_surface_unref (iter->data);
+		g_slist_free (view->filter_surfaces);
+		view->filter_surfaces = NULL;
 	}
-#endif
-
-#if 1
-	cairo_pattern_set_extend (view->pattern_data->pattern, CAIRO_EXTEND_NONE);
-	cairo_set_source (cairo, view->pattern_data->pattern);
-	cairo_paint (cairo);
-#endif
-
-	for (iter = view->filter_surfaces; iter != NULL; iter = iter->next)
-		lsm_filter_surface_free (iter->data);
-	g_slist_free (view->filter_surfaces);
-	view->filter_surfaces = NULL;
 
 	_end_pattern (view);
-#endif
+}
+
+static LsmFilterSurface *
+_get_filter_surface (LsmSvgView *view, const char *input)
+{
+	GSList *iter;
+	LsmFilterSurface *source_surface = NULL;
+
+	if (input == NULL)
+		return view->filter_surfaces->data;
+
+	for (iter = view->filter_surfaces; iter != NULL; iter = iter->next) {
+		LsmFilterSurface *surface = iter->data;
+
+		if (g_strcmp0 (input, lsm_filter_surface_get_name (surface)) == 0)
+			return surface;
+
+		source_surface = surface;
+	}
+
+	if (g_strcmp0 (input, "SourceAlpha") == 0 && source_surface != NULL) {
+		LsmFilterSurface *surface;
+
+		surface = lsm_filter_surface_new_similar ("SourceAlpha", source_surface);
+		lsm_filter_surface_alpha (source_surface, surface);
+		view->filter_surfaces = g_slist_prepend (view->filter_surfaces, surface);	
+
+		return surface;
+	} else if (g_strcmp0 (input, "BackgroundImage") == 0) {
+		/* TODO */
+		LsmFilterSurface *surface;
+
+		surface = lsm_filter_surface_new_similar ("BackgroundImage", source_surface);
+		view->filter_surfaces = g_slist_prepend (view->filter_surfaces, surface);	
+	} else if (g_strcmp0 (input, "BackgroundAlpha") == 0) {
+		/* TODO */
+		LsmFilterSurface *surface;
+
+		surface = lsm_filter_surface_new_similar ("BackgroundAlpha", source_surface);
+		view->filter_surfaces = g_slist_prepend (view->filter_surfaces, surface);	
+	}
+
+	return NULL;
+}
+
+static LsmFilterSurface *
+_create_filter_surface (LsmSvgView *view, const char *output, LsmFilterSurface *input_surface)
+{
+	LsmFilterSurface *surface;
+
+	surface = lsm_filter_surface_new_similar (output, input_surface);
+
+	view->filter_surfaces = g_slist_prepend (view->filter_surfaces, surface); 
+
+	return surface;
+}
+
+void
+lsm_svg_view_apply_blend (LsmSvgView *view, const char *input_1, const char*input_2, const char *output,
+			  LsmSvgBlendingMode mode)
+{
+	LsmFilterSurface *output_surface;
+	LsmFilterSurface *input_1_surface;
+	LsmFilterSurface *input_2_surface;
+
+	g_return_if_fail (LSM_IS_SVG_VIEW (view));
+
+	input_1_surface = _get_filter_surface (view, input_1);
+	input_2_surface = _get_filter_surface (view, input_2);
+
+	if (input_1_surface == NULL || input_2_surface == NULL) {
+		lsm_debug_render ("[SvgView::apply_blend] Inputs '%s' or '%s' not found", input_1, input_2);
+		return;
+	}
+
+	output_surface = _create_filter_surface (view, output, input_1_surface);
+
+	lsm_log_render ("[SvgView::blend] mode = %s", lsm_svg_blending_mode_to_string (mode));
+
+	lsm_filter_surface_blend (input_1_surface, input_2_surface, output_surface, mode);
+}
+
+void
+lsm_svg_view_apply_flood (LsmSvgView *view, const char *output,
+			  double x, double y, double w, double h)
+{
+	LsmFilterSurface *output_surface;
+	LsmFilterSurface *input_surface;
+	guint8 red, green, blue;
+	guint32 color;
+	double opacity;
+
+	g_return_if_fail (LSM_IS_SVG_VIEW (view));
+
+	input_surface = _get_filter_surface (view, NULL);
+
+	output_surface = _create_filter_surface (view, output, input_surface);
+
+	red = (double) (0.5 + view->style->flood_color->value.red * 255.0);
+	green = (double) (0.5 + view->style->flood_color->value.green * 255.0);
+	blue = (double) (0.5 + view->style->flood_color->value.blue * 255.0);
+	color = red << 16 | green << 8 | blue << 0;
+	opacity = view->style->flood_opacity->value;
+
+	lsm_log_render ("[SvgView::apply_flood] color = 0x%06x - opacity = %g",
+			color, opacity);
+
+	lsm_filter_surface_flood (output_surface, color, opacity);
 }
 
 void
@@ -1969,6 +2136,80 @@ lsm_svg_view_apply_gaussian_blur (LsmSvgView *view, const char *input, const cha
 				  double x, double y, double w, double h,
 				  double std_x, double std_y)
 {
+	LsmFilterSurface *input_surface;
+	LsmFilterSurface *output_surface;
+
+	g_return_if_fail (LSM_IS_SVG_VIEW (view));
+
+	input_surface = _get_filter_surface (view, input);
+
+	if (input_surface == NULL) {
+		lsm_debug_render ("[SvgView::apply_gaussian_blur] Input '%s' not found", input);
+		return;
+	}
+
+	output_surface = _create_filter_surface (view, output, input_surface);
+
+	lsm_log_render ("[SvgView::apply_gaussian_blur] %s -> %s (x:%g,y:%g,w:%g,h:%g) (%g,%g)",
+			input != NULL ? input : "previous",
+			output != NULL ? output : "next",
+			x, y, w, h, std_x, std_y); 
+
+	cairo_user_to_device_distance (view->dom_view.cairo, &std_x, &std_y);
+
+	lsm_log_render ("[SvgView::apply_gaussian_blur] %g px,%g px",
+			std_x, std_y);
+
+	lsm_filter_surface_fast_blur (input_surface, output_surface, std_x, std_y);
+}
+
+void
+lsm_svg_view_apply_offset (LsmSvgView *view, const char *input, const char *output, double dx, double dy)
+{
+	LsmFilterSurface *input_surface;
+	LsmFilterSurface *output_surface;
+
+	g_return_if_fail (LSM_IS_SVG_VIEW (view));
+
+	input_surface = _get_filter_surface (view, input);
+
+	if (input_surface == NULL) {
+		lsm_debug_render ("[SvgView::apply_offset] Input '%s' not found", input);
+		return;
+	}
+
+	output_surface = _create_filter_surface (view, output, input_surface);
+
+	lsm_log_render ("[SvgView::apply_offset] %s -> %s (dx:%g,dy:%g)", input, output, dx, dy); 
+
+	cairo_user_to_device_distance (view->dom_view.cairo, &dx, &dy);
+
+	lsm_log_render ("[SvgView::apply_offset] %g px,%g px", dx, dy);
+
+	lsm_filter_surface_offset (input_surface, output_surface, dx, dy);
+}
+
+void
+lsm_svg_view_apply_merge (LsmSvgView *view, const char *input, const char *output)
+{
+	LsmFilterSurface *input_surface;
+	LsmFilterSurface *output_surface;
+
+	g_return_if_fail (LSM_IS_SVG_VIEW (view));
+
+	input_surface = _get_filter_surface (view, input);
+
+	if (input_surface == NULL) {
+		lsm_debug_render ("[SvgView::apply_offset] Input '%s' not found", input);
+		return;
+	}
+
+	output_surface = _get_filter_surface (view, output);
+	if (output_surface == NULL)
+		output_surface = _create_filter_surface (view, output, input_surface);
+
+	if (output_surface != NULL)
+		lsm_filter_surface_merge (input_surface, output_surface);
 }
 
 void
@@ -2007,28 +2248,82 @@ lsm_svg_view_circular_reference_check (LsmSvgView *view, LsmSvgElement *element)
 }
 
 void
-lsm_svg_view_push_style	(LsmSvgView *view, const LsmSvgStyle *style)
+lsm_svg_view_push_style (LsmSvgView *view, LsmSvgStyle *style)
 {
 	g_return_if_fail (LSM_IS_SVG_VIEW (view));
 	g_return_if_fail (style != NULL);
 
+	lsm_log_render ("[SvgView::push_style]");
+
+	if (view->style == NULL || (style->font_size != view->style->font_size)) {
+		LsmSvgViewbox font_viewbox;
+		LsmSvgViewbox *viewbox;
+		double current_font_size_px;
+
+		if (view->style != NULL)
+			current_font_size_px = view->style->font_size_px;
+		else
+			current_font_size_px = 0.0;
+
+		viewbox = view->viewbox_stack->data;
+		font_viewbox.resolution_ppi = viewbox->resolution_ppi;
+		font_viewbox.viewbox.x = 0;
+		font_viewbox.viewbox.y = 0;
+		font_viewbox.viewbox.width = current_font_size_px;
+		font_viewbox.viewbox.height = current_font_size_px;
+
+		style->font_size_px = lsm_svg_length_normalize (&style->font_size->length, &font_viewbox,
+								current_font_size_px, LSM_SVG_LENGTH_DIRECTION_VERTICAL);
+
+		if (style->font_size_px < 0.0)
+			style->font_size_px = 0.0;
+		lsm_log_render ("[SvgView::push_style] Font size = %g pixels", style->font_size_px);
+	} else
+		style->font_size_px = view->style->font_size_px;
+
 	view->style_stack = g_slist_prepend (view->style_stack, (void *) style);
 	view->style = style;
 
-	if (g_strcmp0 (style->filter->value, "none") != 0) {
-		lsm_debug_render ("[LsmSvgView::push_style] Start filter '%s'", style->filter->value);
-		lsm_svg_view_push_filter (view);
-	}
+}
 
-	if (g_strcmp0 (style->clip_path->value, "none") != 0) {
+void
+lsm_svg_view_push_composition (LsmSvgView *view, LsmSvgStyle *style)
+{
+	gboolean do_filter;
+	gboolean do_mask;
+	gboolean do_clip;
+
+	g_return_if_fail (LSM_IS_SVG_VIEW (view));
+	g_return_if_fail (style != NULL);
+
+	lsm_svg_view_push_style (view, style);
+
+	lsm_log_render ("[SvgView::push_composition]");
+
+	do_clip = (g_strcmp0 (style->clip_path->value, "none") != 0);
+	do_mask = (g_strcmp0 (style->mask->value, "none") != 0);
+	do_filter = (g_strcmp0 (style->filter->value, "none") != 0);
+
+	if (view->style->opacity->value < 1.0 && !do_filter)
+		cairo_push_group (view->dom_view.cairo);
+
+	if (do_clip) {
 		lsm_debug_render ("[LsmSvgView::push_style] Start clip '%s'", style->clip_path->value);
 		lsm_svg_view_push_clip (view);
 	}
 
-	if (g_strcmp0 (style->mask->value, "none") != 0) {
+	if (do_mask) {
 		lsm_debug_render ("[LsmSvgView::push_style] Start mask '%s'", style->mask->value);
 		lsm_svg_view_push_mask (view);
 	}
+
+	/* Don't do filtering during a clipping operation, as filter will
+	 * create a new subsurface, where clipping should occur with the path
+	 * of the clip-path element. */ 
+	if (do_filter && !view->is_clipping) {
+		lsm_debug_render ("[LsmSvgView::push_style] Start filter '%s'", style->filter->value);
+		lsm_svg_view_push_filter (view);
+	}
 }
 
 void lsm_svg_view_pop_style (LsmSvgView *view)
@@ -2036,18 +2331,45 @@ void lsm_svg_view_pop_style (LsmSvgView *view)
 	g_return_if_fail (LSM_IS_SVG_VIEW (view));
 	g_return_if_fail (view->style_stack != NULL);
 
-	if (g_strcmp0 (view->style->mask->value, "none") != 0)
-		lsm_svg_view_pop_mask (view);
-
-	if (g_strcmp0 (view->style->clip_path->value, "none") != 0)
-		lsm_svg_view_pop_clip (view);
-
-	if (g_strcmp0 (view->style->filter->value, "none") != 0) {
-		lsm_svg_view_pop_filter (view);
-	}
-
 	view->style_stack = g_slist_delete_link (view->style_stack, view->style_stack);
 	view->style = view->style_stack != NULL ? view->style_stack->data : NULL;
+
+	lsm_log_render ("[SvgView::pop_style]");
+}
+
+void lsm_svg_view_pop_composition (LsmSvgView *view)
+{
+	gboolean do_filter;
+	gboolean do_mask;
+	gboolean do_clip;
+
+	g_return_if_fail (LSM_IS_SVG_VIEW (view));
+	g_return_if_fail (view->style != NULL);
+
+	lsm_log_render ("[SvgView::pop_composition]");
+
+	do_clip = (g_strcmp0 (view->style->clip_path->value, "none") != 0);
+	do_mask = (g_strcmp0 (view->style->mask->value, "none") != 0);
+	do_filter = (g_strcmp0 (view->style->filter->value, "none") != 0);
+
+	/* Don't do filtering during a clipping operation, as filter will
+	 * create a new subsurface, where clipping should occur with the path
+	 * of the clip-path element. */ 
+	if (do_filter && !view->is_clipping)
+		lsm_svg_view_pop_filter (view);
+
+	if (do_mask)
+		lsm_svg_view_pop_mask (view);
+
+	if (do_clip)
+		lsm_svg_view_pop_clip (view);
+
+	if (view->style->opacity->value < 1.0 && !do_filter) {
+		cairo_pop_group_to_source (view->dom_view.cairo);
+		cairo_paint_with_alpha (view->dom_view.cairo, view->style->opacity->value);
+	}
+
+	lsm_svg_view_pop_style (view);
 }
 
 LsmSvgStyle *
@@ -2069,6 +2391,17 @@ lsm_svg_view_get_pattern_extents (LsmSvgView *view)
 	return &view->pattern_data->extents;
 }
 
+const LsmBox *
+lsm_svg_view_get_object_extents (LsmSvgView *view)
+{
+	static LsmBox null_extents = {.x = 0.0, .y = 0.0, .width = 0.0, .height = 0.0};
+
+	g_return_val_if_fail (LSM_IS_SVG_VIEW (view), &null_extents);
+	g_return_val_if_fail (view->pattern_data != NULL, &null_extents);
+
+	return &view->pattern_data->object_extents;
+}
+
 const LsmBox *
 lsm_svg_view_get_clip_extents (LsmSvgView *view)
 {
@@ -2154,6 +2487,19 @@ lsm_svg_view_render (LsmDomView *view)
 	}
 }
 
+static void
+lsm_svg_view_set_debug (LsmDomView *view, const char *feature, gboolean enable)
+{
+	LsmSvgView *svg_view = LSM_SVG_VIEW (view);
+
+	if (g_strcmp0 (feature, "filter") == 0)
+		svg_view->debug_filter = enable;
+	else if (g_strcmp0 (feature, "mask") == 0)
+		svg_view->debug_mask = enable;
+	else if (g_strcmp0 (feature, "pattern") == 0)
+		svg_view->debug_pattern = enable;
+}
+
 LsmSvgView *
 lsm_svg_view_new (LsmSvgDocument *document)
 {
@@ -2169,6 +2515,9 @@ lsm_svg_view_new (LsmSvgDocument *document)
 static void
 lsm_svg_view_init (LsmSvgView *view)
 {
+	view->debug_mask = FALSE;
+	view->debug_filter = FALSE;
+	view->debug_pattern = FALSE;
 }
 
 static void
@@ -2189,6 +2538,7 @@ lsm_svg_view_class_init (LsmSvgViewClass *view_class)
 
 	d_view_class->measure = lsm_svg_view_measure;
 	d_view_class->render = lsm_svg_view_render;
+	d_view_class->set_debug = lsm_svg_view_set_debug;
 }
 
 G_DEFINE_TYPE (LsmSvgView, lsm_svg_view, LSM_TYPE_DOM_VIEW)
